#include <stdio.h>
#include <math.h>

#include "adc.h"
#include "my_i2c.h"
//
int check_acc(int delay, const unsigned int flag);
int check_piezo(int delay, const unsigned int flag);
void acc_exit();
void piezo_exit();

static struct struct_accel *acc;
static struct struct_piezo *piezo;
static struct struct_i2c_comm *i2c;

/* struct_accel 객체생성을 위한 initialize 함수 */
struct struct_accel *acc_init(double xcalib, double ycalib, double zcalib)
{
	acc = (struct struct_accel *)calloc(sizeof(struct struct_accel), 1);

	acc->xcalib = xcalib;
	acc->ycalib = ycalib;
	acc->zcalib = zcalib;

	acc->check_acc = check_acc;
	acc->exit = acc_exit;

	acc->struct_accel_data = (struct struct_accel_data *)calloc(sizeof(struct struct_accel_data), 1);
	i2c = i2c_open();

	return acc;
}

/* struct_piezo 객체생성을 위한 initalize 함수 */
struct struct_piezo *piezo_init(unsigned short lowlimit, unsigned short highlimit)
{
	piezo = (struct struct_piezo *)calloc(sizeof(struct struct_piezo), 1);

	piezo->lowlimit = lowlimit;
	piezo->highlimit = highlimit;

	piezo->check_piezo = check_piezo;
	piezo->exit = piezo_exit;

	i2c = i2c_open();

	return piezo;
}


/* struct_accel 소멸 */
void acc_exit()
{
	i2c->close();
	free(acc->struct_accel_data);
	free(acc);
}

/* struct_piezo 소멸 */
void piezo_exit()
{
	i2c->close();
	free(piezo);
}
/* 가속도계에서 i2c를통해 값을 읽어오고 결과를 struct_accel_data에 저장 */
static void _read_from_adc(int delay, const unsigned int flag)
{
	int a = 0;
	unsigned short var = 0;
	double x, y, z;


	i2c->read(delay, flag);

	switch (flag & ADC_SEL_MASK) {
		case ADC_ACCEL:
		{
			struct struct_accel_data *data;
			data = acc->struct_accel_data;
			for (a=0; a<3; a++) {
				var = i2c->read_buf.acc[a][1];
				var |= (i2c->read_buf.acc[a][0] << 8) & ~0x00ff;

				if (a == 0)
					data->rawx = var;
				else if (a == 1)
					data->rawy = var;
				else
					data->rawz = var;
			}
		}
		break;
		case ADC_PIEZO:
			var = i2c->read_buf.piezo[1];
			var |= (i2c->read_buf.piezo[0] << 8) & ~0x00ff;
			piezo->raw = var;
			break;

		default:	
	}
}

/* 가속도 관련 연산 */
static void _calc_acc()
{
	double result;
	struct struct_accel_data *data;

	data = acc->struct_accel_data;

	/* 가속도센서 리턴값에 1비트당 전압을 곱하여 디지털전압을 구해 struct_accel에 저장 */
	data->dxv = data->rawx * ACC_BPV;
	data->dyv = data->rawy * ACC_BPV;
	data->dzv = data->rawz * ACC_BPV;

	/* 구해진 디지털전압을 통해 칼리브레이션을 적용하여 중력가속도를 구한다 */
	data->xacc = (data->dxv - (1.65 + acc->xcalib)) / ACC_SENS;
	data->yacc = (data->dyv - (1.65 + acc->ycalib)) / ACC_SENS;
	data->zacc = (data->dzv - (1.65 + acc->zcalib)) / ACC_SENS;

	/* 중력가속도를 기반으로 실제 각도를 산출 */
	result = atan(data->xacc / sqrt(pow(data->yacc, 2)+pow(data->zacc, 2)));
	data->xtheta = rad2theta(result);

	result = atan(data->yacc / sqrt(pow(data->xacc, 2)+pow(data->zacc, 2)));
	data->ytheta = rad2theta(result);

	result = atan(sqrt(pow(data->xacc, 2)+pow(data->yacc, 2)) / data->zacc);
	data->ztheta = rad2theta(result);
}



/* 체크 함수 모음 */
/* check_adc함수에서 적절한 체크함수를 호출시킨다 */



/* 흔들기 감지 */
/* return 0 ==> 행동없음 */
/* return 1 ==> 흔들기있음 */
static int _check_shake(int delay, const unsigned int flag)
{
	static unsigned int count = 0;
	struct struct_accel_data *data;

	data = acc->struct_accel_data;

	_read_from_adc(delay, flag);
	_calc_acc();

	if (data->rawx < 1000 || data->rawy < 1000 || data->rawz < 1000) {
		if (count == 1)
		{
			count=0;
			return 1;
		}else
			count++;
	} else
		return 0;
	
	return 0;
}

/* 방향 감지 */
/* return 0 ==> 행동없음 */
/* return 1 ==> 왼쪽이동 */
/* return 2 ==> 오른쪽이동 */
/* return 3 ==> 위쪽이동 */
/* return 4 ==> 아래쪽이동 */
static int _check_direction(int delay, const unsigned int flag)
{
	static unsigned int ucount, dcount, rcount, lcount;
	struct struct_accel_data *data;

	data = acc->struct_accel_data;

	_read_from_adc(delay, flag);
	_calc_acc();

	if (data->yacc > 1.5 && data->yacc < 3)
		return 1;
	else if (data->yacc < -2 && data->yacc > -3)
		return 2;
	else if (data->xacc < -1.8 && data->xacc > -2)
		return 3;
	else if (data->xacc > 1 && data->xacc < 3)
		return 4;
	else
		return 0;
	
	return 0;
}




/* 충격 감지 */
/* return 0 ==> 행동없음 */
/* return 1 ==> 충격있음 */
static int _check_shock(int delay, const unsigned int flag)
{
	struct struct_accel_data *data;

	data = acc->struct_accel_data;

	_read_from_adc(delay, flag);
	_calc_acc();

	if (data->ztheta >= 15)
		return 1;
	else
		return 0;
	return 0;
}

/* 붓기 감지 */
/* return 0 ==> 행동없음 */
/* return 1 ==> 왼쪽으로 붓기 */
/* return 2 ==> 오른쪽으로 붓기 */
/* return 3 ==> 위쪽으로 붓기 */
/* return 4 ==> 아랫쪽으로 붓기 */
static int _check_drop(int delay, const unsigned int flag)
{
	struct struct_accel_data *data;

	data = acc->struct_accel_data;

	_read_from_adc(delay, flag);
	_calc_acc();

	if ((data->xtheta >= -10 && data->xtheta <= 10) &&
			(data->ytheta >= -70 && data->ytheta <= -20) &&
			(data->ztheta >= 20 && data->ztheta <= 70)) {

		return 1;
	} else if ((data->xtheta >= -20 && data->xtheta <= 20) &&
			(data->ytheta >= 20 && data->ytheta <= 70) &&
			(data->ztheta >= 20 && data->ztheta <= 70)) {

		return 2;
	} else if ((data->xtheta >= 30 && data->xtheta <= 60) &&
			(data->ytheta >= -10 && data->ytheta <= 10) &&
			(data->ztheta >= 30 && data->ztheta <= 60)) {

		return 3;
	} else if ((data->xtheta >= -70 && data->xtheta <= -20) &&
			(data->ytheta >= -10 && data->ytheta <= 10) &&
			(data->ztheta >= 20 && data->ztheta <= 70)) {

		return 4;
	} else {

		return 0;
	}

	return 0;
}

/* 메뉴 이동 기울이기 */
static int _check_menu_drop(int delay, const unsigned int flag)
{
	struct struct_accel_data *data;

	data = acc->struct_accel_data;

	_read_from_adc(delay, flag);
	_calc_acc();

	if ((data->xtheta >= -10 && data->xtheta <= 10) &&
			(data->ytheta >= -70 && data->ytheta <= -10) &&
			(data->ztheta >= 15 && data->ztheta <= 70)) {

		return 1;
	} else if ((data->xtheta >= -10 && data->xtheta <= 10) &&
			(data->ytheta >= 10 && data->ytheta <= 70) &&
			(data->ztheta >= 15 && data->ztheta <= 70)) {

		return 2;
	} else if ((data->xtheta >= 10 && data->xtheta <= 70) &&
			(data->ytheta >= -10 && data->ytheta <= 10) &&
			(data->ztheta >= 10 && data->ztheta <= 70)) {

		return 3;
	} else if ((data->xtheta >= -70 && data->xtheta <= -10) &&
			(data->ytheta >= -10 && data->ytheta <= 10) &&
			(data->ztheta >= 20 && data->ztheta <= 70)) {

		return 4;
	} else {

		return 0;
	}

	return 0;
}



/* piezo 센서 관련 */

int _check_wind(int delay, const unsigned int flag)
{
	_read_from_adc(delay, flag);

	if (piezo->raw <= piezo->lowlimit || piezo->raw >= piezo->highlimit)
		return 1;
	return 0;
}
	

/* 사용자가 콜한 check_adc 함수를 플래그 행동양식에 맞게 연결후 호출 */
int check_acc(int delay, const unsigned int flag)
{
	int ret;
	int (*exec_func)(int, const unsigned int) = NULL;

	switch(flag & ACC_SEL_MASK) {
		case ACC_SHOCK:
			exec_func = _check_shock;
			break;
		case ACC_DROP:
			exec_func = _check_drop;
			break;
		case ACC_SHAKE:
			exec_func = _check_shake;
			break;
		case ACC_DIRECTION:
			exec_func = _check_direction;
			break;
		case ACC_MENU_DROP:
			exec_func = _check_menu_drop;
		default:
	}
	if (exec_func)
		ret = exec_func(delay, flag);

	return ret;
}

int check_piezo(int delay, const unsigned int flag)
{
	int ret;
	int (*exec_func)(int, const unsigned int) = NULL;

	switch(flag & PIEZO_SEL_MASK) {
		case PIEZO_WIND:
			exec_func = _check_wind;
			break;
		default:
	}
	if (exec_func)
		ret = exec_func(delay, flag);

	return ret;
}
