#ifndef __KERNEL__
#  define __KERNEL__
#endif

#ifndef MODULE
#  define MODULE
#endif

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>    
#include <linux/version.h>
#include <linux/slab.h>      
#include <linux/fs.h>        
#include <linux/errno.h>     
#include <linux/types.h>     
#include <linux/proc_fs.h>
#include <linux/fcntl.h>     
#include <linux/ioport.h>
#include <linux/vmalloc.h>
#include <linux/ioctl.h>
#include <linux/sched.h> 
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/major.h>
#include <linux/delay.h>
#include <linux/string.h>
#include <linux/signal.h>
#include <linux/ioport.h>
#include <linux/mm.h>
#include <linux/init.h>

#include <asm/system.h>      
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/hardware.h>
#include <asm/arch-pxa/pxa-regs.h>

#include "irda.h"

/* Set of debugging defines */
#define GPIO_DEBUG
#undef  GPIO_DEBUG_NO

DECLARE_WAIT_QUEUE_HEAD(gpio_wait);

static int gpio_usage=0;
static int set_direction=0;

/*************************************************************************************************
 * 
 * void GPIO_IO_Init(void)
 *
 * - 사용하고자 하는 GPIO를 입력 혹은 출력, 얼터네이티브 기능 사용/비사용으로 설정
 *   	
 *   	@ 해당 비트를 1로 만들고자 한다면, 해당 GPIO 레지스터에 비트를 OR연산한다
 *   		ex) GPDR0 |= 0x000000001 // GPIO 1번에 해당하는 GPDR0의 값을 1로 셋팅
 *   	
 *   	@ 해당 비트를 0으로 만들고자 한다면, 해당 GPIO 레지스터에 비트를 AND NOT 연산을 한다
 *   		ex) GPDR0 &= ~(0x000000001) // GPIO 1번에 해당하는 GPDR0의 값을 0으로 셋팅
 *
 *************************************************************************************************/
void GPIO_IO_Init(void) 
{
	GAFR0_L &= 0xC00FFFFF;		// GPIO 10, 11, 12, 13, 14의 얼터네이티브 기능을 OFF한다
	GPDR0	|= MASK_GPIO_TX;	// GPIO 10번(TX)을 출력 방향으로 셋팅
	GPDR0	&= ~(ALL_RX_MASK);	// GPIO 11, 12, 13, 14번(RX)을 입력 방향으로 셋팅
}

int tx(void)
{
	GPSR0	|= MASK_GPIO_TX;

	return 1;
}

int rx(void)
{
	switch(set_direction)
	{
		case SET_RX_DOWN:
			
			return ((GPLR0 & MASK_GPIO_RX_DOWN) >> GPIO_RX_DOWN);

		case SET_RX_LEFT:

			return ((GPLR0 & MASK_GPIO_RX_LEFT) >> GPIO_RX_LEFT);

		case SET_RX_UP:

			return ((GPLR0 & MASK_GPIO_RX_UP) >> GPIO_RX_UP);

		case SET_RX_RIGHT:

			return ((GPLR0 & MASK_GPIO_RX_RIGHT) >> GPIO_RX_RIGHT);
	}

	return -1;
}

ssize_t irda_read(struct file *filp, char *buf, size_t count, loff_t *f_pos)
{
  	return rx();
}

ssize_t irda_write(struct file *filp, const char *buf, size_t count, loff_t *f_pos)
{
	return tx();
}

int irda_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
	switch(cmd)
	{
		case SET_RX_DOWN:

			set_direction=SET_RX_DOWN;
			break;

		case SET_RX_LEFT:

			set_direction=SET_RX_LEFT;
			break;

		case SET_RX_UP:

			set_direction=SET_RX_UP;
			break;

		case SET_RX_RIGHT:

			set_direction=SET_RX_RIGHT;
			break;

		case IRDA_WRITE:

			return tx();

		case IRDA_READ:

			return rx();
	}

	return 0;
}

int irda_open(struct inode *inode, struct file *filp)
{
	if(gpio_usage != 0) return -EBUSY;
  	
	MOD_INC_USE_COUNT; 

	gpio_usage=1; 

	return 0;           
}

int irda_release(struct inode *inode, struct file *filp)
{
	MOD_DEC_USE_COUNT;  

	gpio_usage=0;

	GPCR0	|= MASK_GPIO_TX;
	
	return 0;
}

static struct file_operations relay_fops = 
{
	read	:	irda_read,
	write	:	irda_write,
	ioctl	:	irda_ioctl,
	open	:	irda_open,
	release	:	irda_release,
};

static int __init irda_init_module(void)
{
	int result;
    
	result=register_chrdev( IRDA_MAJOR, DEVICE_NAME, &relay_fops);
	if (result < 0) 
	{
		printk(KERN_WARNING "\n%s : Can't get Major Number [%d]\n", DEVICE_NAME, IRDA_MAJOR);
		return result;
	}
	printk("\n\n\nInit madule, Succeed. This Device is %s and Major Number is [%d]\n\n", DEVICE_NAME, IRDA_MAJOR);

	GPIO_IO_Init();

	return 0;
}

static void irda_cleanup_module(void)
{
	if (!unregister_chrdev(IRDA_MAJOR, DEVICE_NAME)) 
		printk("%s Device Exit Sucess...\n", DEVICE_NAME);
	else		
		printk("%s Device Exit Fail...\n", DEVICE_NAME);
}

module_init(irda_init_module);
module_exit(irda_cleanup_module);
